import * as mysql from "mysql";

import { IDBDriver } from "./db-driver.interface";
import { IMigration } from "./migrations.interface";

export class MySQLDriver implements IDBDriver {
  private connection: undefined | mysql.Connection;
  constructor(private settings: IMigration) {}

  static driverName = "MySQL";

  async connect(): Promise<void> {
    this.connection = mysql.createConnection({
      host: this.settings.host,
      user: this.settings.user,
      password: this.settings.password,
      database: this.settings.database,
      port: this.settings.port,
    });

    return new Promise((res, rej) => {
      this.connection?.connect((error, args) => {
        if (error) {
          rej(error);
        }
        res(args);
      });
    }) as Promise<void>;
  }

  async close(): Promise<void> {
    return new Promise((res, rej) => {
      this.connection?.end((error) => {
        if (error) {
          rej(error);
        }
        res();
      });
    }) as Promise<void>;
  }

  async dispatch<T>(query: string): Promise<any> {
    return new Promise((res, rej) => {
      this.connection?.query(query, (error, results, fields) => {
        if (error) {
          return rej(error);
        }
        res(results);
      });
    });
  }
}

export const DRIVERS: { [key: string]: IDBDriver }[] = [];
