import * as yaml from "js-yaml";
import * as fs from "fs";
import * as path from "path";
import { IMigration, IMigrationsConfig } from "./migrations.interface";
import { Migration } from "./migration";
import { Command } from "./command";
import { Reader } from "./reader";
import * as colors from "colors";
import { DriversInfo } from "./drivers-info";

const main = async () => {
  colors.enable();

  const { path: route, file } = new Command().dispatch();

  let migrations: Migration;

  const configPath = path.join(process.cwd(), `/${route}/${file}`);
  migrations = new Migration(
    yaml.load(fs.readFileSync(configPath, "utf8")) as IMigrationsConfig
  );

  for (const migration of migrations.toPrimitives()) {
    let {
      extension,
      folder,
      file: _file,
      recursive,
      type,
      migrationName,
    } = migration;

    console.log(`\nRunning a new Migration: ${migrationName}`.bold);

    const migrationInfo = DriversInfo[type];

    if (!migrationInfo) {
      throw new Error(`There are NOT migration for type: ${type}`);
    }

    if (!extension) {
      extension = migrationInfo.extension;
    }

    const driver = new migrationInfo.driver(migration);

    await driver.connect();

    const reader = new Reader(driver);

    await reader.readFiles(
      `${folder}${_file ? _file : ""}`,
      extension,
      !!recursive
    );

    await driver.close();
  }
};

main().catch((err) => {
  console.error(err);
});
