import { IMigration, IMigrationsConfig } from "./migrations.interface";

// At this moment only support "mysql"
export class Migration {
  constructor(private config: IMigrationsConfig) {
    this.validateConfig();
  }

  private validateConfig(): void {
    if (!this.config || !this.config.migrations) {
      throw new Error("Config and at least one migration must be defined.");
    }

    !this.config.name && (this.config.name = "Default Name");

    !this.config.directory && (this.config.directory = "migrations");
  }

  private validateMigration({
    type,
    host,
    port,
    password,
    database,
    user,
    extension,
    file,
    folder,
    migrationName = "Default Migration",
    recursive,
  }: IMigration): IMigration {
    if (type !== "mysql") {
      throw new Error("Only support mysql like migration type.");
    }

    if (!host) {
      throw new Error("You must define a migration URL or a migration HOST.");
    }

    return {
      database,
      port,
      password,
      host,
      user,
      type,
      extension,
      file,
      folder,
      migrationName,
      recursive,
    };
  }

  public toPrimitives(): IMigration[] {
    if (!this.config?.migrations?.length) {
      throw new Error("Config and at least one migration must be defined.".red);
    }

    return this.config.migrations.map((migration) =>
      this.validateMigration(migration.migration)
    );
  }
}
