import * as fs from "fs";
import * as path from "path";
import { IDBDriver } from "./db-driver.interface";

export class Reader {
  private recursive = false;
  private regEx: null | RegExp = null;
  constructor(private driver: IDBDriver) {}

  public readFiles = async (
    route: string,
    extension: string,
    recursive: boolean
  ) => {
    this.recursive = recursive;
    this.regEx = new RegExp(`.${extension}$`);
    console.log(route, extension, recursive);
    const _idDir = await this.isDir(route);
    if (this.regEx.test(route) && !_idDir) {
      const content = await this.read(route);
      this.driver.dispatch(content);
    }
    if (_idDir && recursive) {
      // const files = this.readDir(route);
      try {
        await this.readAll(route);
      } catch (error) {
        console.log("MMMMMMMMMMMMMMM");
      }
    }
  };

  private isDir = async (route: string): Promise<boolean> => {
    const stat = await fs.promises.lstat(route);
    return stat.isDirectory();
  };

  private hasExtension(file: string): boolean {
    return this.regEx!.test(file);
  }

  private read = async (fileName: string): Promise<string> => {
    return fs.promises.readFile(fileName, "utf8");
  };

  private readDir = async (route: string): Promise<string[]> =>
    fs.promises.readdir(path.join(process.cwd(), route));

  private readAll = async (route: string) => {
    try {
      const routeIsDir = await this.isDir(route);
      if (routeIsDir) {
        const dirs = await this.readDir(route);

        for (let entry of dirs) {
          const file = `${route}/${entry}`;
          const entryIsDir = await this.isDir(file);
          if (entryIsDir && this.recursive) {
            await this.readAll(file);
            return;
          }

          if (this.hasExtension(entry)) {
            const content = await this.read(file);

            try {
              console.log(`Running file: ${file.bold}`.green);
              await this.driver.dispatch(content!);
            } catch (error) {
              console.log(`Error running file: ${file.bold}`.red);
              console.error(error);
            }
          } else {
            console.log(`Excluded file: ${file.bold}`.yellow);
          }
        }
      }
    } catch (error) {
      console.error(error);
    }
  };
}
